<?php

declare(strict_types=1);

namespace spec\Dedi\SyliusSogenactifPlugin\Payum\Sogenactif\Action;

use Sylius\Component\Core\Model\OrderInterface;
use Dedi\SyliusSogenactifPlugin\Payum\Sogenactif\ApiInterface;
use Dedi\SyliusSogenactifPlugin\Payum\Sogenactif\Generator\TransactionInterface;
use Dedi\SyliusSogenactifPlugin\Payum\Sogenactif\SealEncoderInterface;
use GuzzleHttp\Client;
use Payum\Core\Action\ActionInterface;
use Payum\Core\ApiAwareInterface;
use Payum\Core\Exception\UnsupportedApiException;
use Payum\Core\Reply\HttpPostRedirect;
use Payum\Core\Request\Capture;
use PhpSpec\ObjectBehavior;
use Prophecy\Argument;
use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\StreamInterface;
use Sylius\Bundle\PayumBundle\Model\PaymentSecurityTokenInterface;
use Sylius\Component\Core\Model\PaymentInterface as SyliusPaymentInterface;
use Symfony\Component\HttpFoundation\ParameterBag;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\RequestStack;

final class CaptureSpec extends ObjectBehavior
{
    function let(
        Client $client,
        SealEncoderInterface $sealEncoder,
        RequestStack $requestStack,
        TransactionInterface $transaction
    )
    {
        $this->beConstructedWith(
            $client,
            $sealEncoder,
            $requestStack,
            $transaction,
            ''
        );
    }

    function it_should_be_able_to_do_action_for_payment()
    {
        $this->shouldImplement(ActionInterface::class);
        $this->shouldImplement(ApiAwareInterface::class);
    }

    function it_sogenactif_api_should_be_accepted(ApiInterface $sogenactifApi)
    {
        $this->setApi($sogenactifApi);

        $this->shouldThrow(UnsupportedApiException::class)
            ->during('setApi', [Argument::any()])
        ;
    }

    function it_could_throw_a_redirection_when_user_call_for_first_time_paiment(
        RequestStack $requestStack,
        Capture $payumRequestCapture,
        Client $client,
        SyliusPaymentInterface $payment,
        PaymentSecurityTokenInterface $paymentSecurityToken,
        ResponseInterface $responseMessage,
        Request $httpRequest,
        ParameterBag $bag,
        SealEncoderInterface $sealEncoder,
        OrderInterface $order,
        ApiInterface $sogenactifApi,
        StreamInterface $stream,
        TransactionInterface $transaction
    )
    {
        $sogenactifApi
            ->getMerchantId()
            ->shouldBeCalledOnce()
            ->willReturn('merchantId')
        ;
        $sogenactifApi
            ->getSecretKey()
            ->shouldBeCalledOnce()
            ->willReturn('secret')
        ;
        $sogenactifApi
            ->getKeyVersion()
            ->shouldBeCalledOnce()
            ->willReturn(1)
        ;

        $sogenactifApi
            ->getKeyVersion()
            ->shouldBeCalledOnce()
            ->willReturn('1234')
        ;

        $this->setApi($sogenactifApi);

        // GIVEN
        $transaction
            ->generate()
            ->shouldBeCalledOnce()
        ;

        $payumRequestCapture
            ->getModel()
            ->shouldBeCalledTimes(2)
            ->willReturn($payment)
        ;

        $payumRequestCapture
            ->getToken()
            ->shouldBeCalledOnce()
            ->willReturn($paymentSecurityToken)
        ;

        $requestStack
            ->getCurrentRequest()
            ->shouldBeCalledOnce()
            ->willReturn($httpRequest)
        ;

        $paymentSecurityToken
            ->getTargetUrl()
            ->shouldBeCalledOnce()
            ->willReturn('targetUrl')
        ;

        $payment
            ->getOrder()
            ->shouldBeCalledOnce()
            ->willReturn($order)
        ;

        $order
            ->getTotal()
            ->shouldBeCalledOnce()
            ->willReturn(123)
        ;


        $httpRequest->request = $bag;

        $bag
            ->get('Data')
            ->shouldBeCalledOnce()
            ->willReturn(null)
        ;

        $contentResponse = <<<'JSON'
{
    "redirectionUrl" : "foo",
    "redirectionVersion" : "bar",
    "redirectionData" : "bob"
}
JSON;

        $stream
            ->getContents()
            ->shouldBeCalledOnce()
            ->willReturn($contentResponse)
        ;

        $responseMessage
            ->getBody()
            ->shouldBeCalledOnce()
            ->willReturn($stream)
        ;

        $client
            ->post(Argument::any(), Argument::any())
            ->shouldBeCalledOnce()
            ->willReturn($responseMessage)
        ;

        $payment
            ->setDetails(["status" => "processing"])
            ->shouldBeCalledOnce()
        ;

        // THEN
        $sealEncoder
            ->createSeal(Argument::any(), Argument::any())
            ->shouldBeCalledOnce()
        ;

        // IF GOOD GO TO REDIRECT
        $this->shouldThrow(HttpPostRedirect::class)
            ->during('execute', [$payumRequestCapture])
        ;
    }

    function it_handle_request_after_redirection(
        RequestStack $requestStack,
        Capture $payumRequestCapture,
        SyliusPaymentInterface $payment,
        PaymentSecurityTokenInterface $paymentSecurityToken,
        Request $httpRequest,
        ParameterBag $bag,
        Client $client,
        SealEncoderInterface $sealEncoder
    )
    {
        $payumRequestCapture
            ->getModel()
            ->shouldBeCalledTimes(2)
            ->willReturn($payment)
        ;

        $payumRequestCapture
            ->getToken()
            ->shouldBeCalledOnce()
            ->willReturn($paymentSecurityToken)
        ;

        $requestStack
            ->getCurrentRequest()
            ->shouldBeCalledOnce()
            ->willReturn($httpRequest)
        ;

        $httpRequest->request = $bag;

        $bag
            ->get('Data')
            ->shouldBeCalledOnce()
            ->willReturn('foo')
        ;

        $this->shouldNotThrow(new HttpPostRedirect(Argument::any()))
            ->during('execute', [$payumRequestCapture]);
    }

    function it_handle_data_empty_from_payback_and_store_everytime_all_request_data(
        RequestStack $requestStack,
        Capture $payumRequestCapture,
        SyliusPaymentInterface $payment,
        PaymentSecurityTokenInterface $paymentSecurityToken,
        Request $httpRequest,
        ParameterBag $bag,
        Client $client,
        SealEncoderInterface $sealEncoder
    )
    {
        $payumRequestCapture
            ->getModel()
            ->shouldBeCalledTimes(2)
            ->willReturn($payment)
        ;

        $payumRequestCapture
            ->getToken()
            ->shouldBeCalledOnce()
            ->willReturn($paymentSecurityToken)
        ;

        $requestStack
            ->getCurrentRequest()
            ->shouldBeCalledOnce()
            ->willReturn($httpRequest)
        ;

        $httpRequest->request = $bag;

        $bag
            ->get('Data')
            ->shouldBeCalledOnce()
            ->willReturn('')
        ;

        $payment
            ->setDetails([
                "error_data_from_return" => ["No data, maybe bad separator. Should be like `Key=Value|Key=Value` but data receive is "],
                "request_data_from_payback" => "",
                "all_normalized_infos" => []
            ])
            ->shouldBeCalledOnce()
        ;

        $this->execute($payumRequestCapture);
    }

    function it_handle_data_bad_formatted_from_payback_and_store_everytime_all_request_data(
        RequestStack $requestStack,
        Capture $payumRequestCapture,
        SyliusPaymentInterface $payment,
        PaymentSecurityTokenInterface $paymentSecurityToken,
        Request $httpRequest,
        ParameterBag $bag,
        Client $client,
        SealEncoderInterface $sealEncoder
    )
    {
        $payumRequestCapture
            ->getModel()
            ->shouldBeCalledTimes(2)
            ->willReturn($payment)
        ;

        $payumRequestCapture
            ->getToken()
            ->shouldBeCalledOnce()
            ->willReturn($paymentSecurityToken)
        ;

        $requestStack
            ->getCurrentRequest()
            ->shouldBeCalledOnce()
            ->willReturn($httpRequest)
        ;

        $httpRequest->request = $bag;

        $bag
            ->get('Data')
            ->shouldBeCalledOnce()
            ->willReturn('foo=bar|alice=bob|duck|test')
        ;

        $payment
            ->setDetails([
                "error_data_from_return" => [
                    "No data, maybe bad separator. Should be like `Key=Value|Key=Value` but data receive is duck",
                    "No data, maybe bad separator. Should be like `Key=Value|Key=Value` but data receive is test"],
                "request_data_from_payback" => "foo=bar|alice=bob|duck|test",
                "all_normalized_infos" => [
                    "foo" => "bar",
                    "alice" => "bob"
                ]
            ])
            ->shouldBeCalledOnce()
        ;

        $this->execute($payumRequestCapture);
    }

    function it_handle_data_bad_formatted_with_no_equals_from_payback_and_store_everytime_all_request_data(
        RequestStack $requestStack,
        Capture $payumRequestCapture,
        SyliusPaymentInterface $payment,
        PaymentSecurityTokenInterface $paymentSecurityToken,
        Request $httpRequest,
        ParameterBag $bag,
        Client $client,
        SealEncoderInterface $sealEncoder
    )
    {
        $payumRequestCapture
            ->getModel()
            ->shouldBeCalledTimes(2)
            ->willReturn($payment)
        ;

        $payumRequestCapture
            ->getToken()
            ->shouldBeCalledOnce()
            ->willReturn($paymentSecurityToken)
        ;

        $requestStack
            ->getCurrentRequest()
            ->shouldBeCalledOnce()
            ->willReturn($httpRequest)
        ;

        $httpRequest->request = $bag;

        $bag
            ->get('Data')
            ->shouldBeCalledOnce()
            ->willReturn('foo=bar|alice=bob||test')
        ;

        $payment
            ->setDetails([
                "error_data_from_return" => [
                    "No data, maybe bad separator. Should be like `Key=Value|Key=Value` but data receive is ",
                    "No data, maybe bad separator. Should be like `Key=Value|Key=Value` but data receive is test"],
                "request_data_from_payback" => "foo=bar|alice=bob||test",
                "all_normalized_infos" => [
                    "foo" => "bar",
                    "alice" => "bob"
                ]
            ])
            ->shouldBeCalledOnce()
        ;

        $this->execute($payumRequestCapture);
    }
}
